/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.
  
    This file is part of the TerraLib - a Framework for building GIS enabled applications.
    
    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.
    
    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */


//Terralib
#include "RasterTransformWizardPage.h"

#include "../../../datatype/Enums.h"
#include "../../../dataaccess/utils/Utils.h"
#include "../../../raster/Band.h"
#include "../../../raster/BandProperty.h"
#include "../../widgets/srs/SRSManagerDialog.h"

#include <QMessageBox>
#include <QListWidgetItem>
#include <QDoubleValidator>
#include <QIntValidator>

// STL


te::qt::widgets::RasterTransformWizardPage::RasterTransformWizardPage(QWidget *parent) :
  QWizardPage(parent)
{
  m_uiPageForm.reset(new Ui::RasterTransformWizardPageForm);
  m_uiPageForm->setupUi(this);

  this->setTitle(tr("Raster Transform"));
  this->setSubTitle(tr("Raster data transform"));
  
  // fill data types combo
  
  m_uiPageForm->m_outputDatatypeComboBox->addItem( "CHAR_TYPE", 
    QVariant( (unsigned int)te::dt::CHAR_TYPE ) );
  m_uiPageForm->m_outputDatatypeComboBox->addItem( "UCHAR_TYPE", 
    QVariant( (unsigned int)te::dt::UCHAR_TYPE ) );
  m_uiPageForm->m_outputDatatypeComboBox->addItem( "INT16_TYPE", 
    QVariant( (unsigned int)te::dt::INT16_TYPE ) );
  m_uiPageForm->m_outputDatatypeComboBox->addItem( "UINT16_TYPE", 
    QVariant( (unsigned int)te::dt::UINT16_TYPE ) );  
  m_uiPageForm->m_outputDatatypeComboBox->addItem( "INT32_TYPE", 
    QVariant( (unsigned int)te::dt::INT32_TYPE ) );
  m_uiPageForm->m_outputDatatypeComboBox->addItem( "UINT32_TYPE", 
    QVariant( (unsigned int)te::dt::UINT32_TYPE ) );
  m_uiPageForm->m_outputDatatypeComboBox->addItem( "FLOAT_TYPE", 
    QVariant( (unsigned int)te::dt::FLOAT_TYPE ) );
  m_uiPageForm->m_outputDatatypeComboBox->addItem( "DOUBLE_TYPE", 
    QVariant( (unsigned int)te::dt::DOUBLE_TYPE ) );
  m_uiPageForm->m_outputDatatypeComboBox->addItem( "CINT16_TYPE", 
    QVariant( (unsigned int)te::dt::CINT16_TYPE ) );
  m_uiPageForm->m_outputDatatypeComboBox->addItem( "CINT32_TYPE", 
    QVariant( (unsigned int)te::dt::CINT32_TYPE ) );
  m_uiPageForm->m_outputDatatypeComboBox->addItem( "CFLOAT_TYPE", 
    QVariant( (unsigned int)te::dt::CFLOAT_TYPE ) );
  m_uiPageForm->m_outputDatatypeComboBox->addItem( "CDOUBLE_TYPE", 
    QVariant( (unsigned int)te::dt::CDOUBLE_TYPE ) );
 
  // fill interpolator combo
  
  m_uiPageForm->m_interpComboBox->addItem( "NearestNeighbor", QVariant( 
    (unsigned int)te::rst::NearestNeighbor ) );
  m_uiPageForm->m_interpComboBox->addItem( "Bilinear", QVariant( 
    (unsigned int)te::rst::Bilinear ) );
  m_uiPageForm->m_interpComboBox->addItem( "Bicubic", QVariant( 
    (unsigned int)te::rst::Bicubic ) );
  
  // validators
  
  QDoubleValidator* oldNoDataValueLineEditValidator = new QDoubleValidator( this );
  m_uiPageForm->m_oldNoDataValueLineEdit->setValidator( oldNoDataValueLineEditValidator );
  
  QDoubleValidator* newNoDataValueLineEditValidator = new QDoubleValidator( this );
  m_uiPageForm->m_newNoDataValueLineEdit->setValidator( newNoDataValueLineEditValidator );  
  
  QDoubleValidator* ressampleXScaleLinesEditValidator = new QDoubleValidator( this );
  ressampleXScaleLinesEditValidator->setBottom( 0 );
  m_uiPageForm->m_ressampleXScaleLinesEdit->setValidator( ressampleXScaleLinesEditValidator ); 

  QDoubleValidator* ressampleYScaleLinesEditValidator = new QDoubleValidator( this );
  ressampleYScaleLinesEditValidator->setBottom( 0 );
  m_uiPageForm->m_ressampleYScaleLinesEdit->setValidator( ressampleYScaleLinesEditValidator ); 

  QIntValidator* ressampleLinesEditValidator = new QIntValidator( this );
  ressampleLinesEditValidator->setBottom( 1 );
  m_uiPageForm->m_ressampleLinesEdit->setValidator( ressampleLinesEditValidator ); 

  QIntValidator* ressampleColsLineEditValidator = new QIntValidator( this );
  ressampleColsLineEditValidator->setBottom( 1 );
  m_uiPageForm->m_ressampleColsLineEdit->setValidator( ressampleColsLineEditValidator ); 

  QDoubleValidator* ressampleResolutionLXLineEditValidator = new QDoubleValidator( this );
  ressampleResolutionLXLineEditValidator->setBottom( 0 );
  m_uiPageForm->m_ressampleResolutionLXLineEdit->setValidator( ressampleResolutionLXLineEditValidator ); 

  QDoubleValidator* ressampleResolutionLYLineEditValidator = new QDoubleValidator( this );
  ressampleResolutionLYLineEditValidator->setBottom( 0 );
  m_uiPageForm->m_ressampleResolutionLYLineEdit->setValidator( ressampleResolutionLYLineEditValidator );  
  
  QIntValidator* oldSRSLineEditValidator = new QIntValidator( this );
  oldSRSLineEditValidator->setBottom( 0 );
  m_uiPageForm->m_oldSRSLineEdit->setValidator( oldSRSLineEditValidator ); 

  QIntValidator* newSRSLineEditValidator = new QIntValidator( this );
  newSRSLineEditValidator->setBottom( 1 );
  m_uiPageForm->m_newSRSLineEdit->setValidator( newSRSLineEditValidator );   
  
  // connections
  
  connect(m_uiPageForm->m_selectOldSRSPushButton, SIGNAL(clicked()), this, 
    SLOT(onSelectOldSRSPushButtonClicked()));
  connect(m_uiPageForm->m_selectNewSRSPushButton, SIGNAL(clicked()), this, 
    SLOT(onSelectNewSRSPushButtonClicked()));   
}

te::qt::widgets::RasterTransformWizardPage::~RasterTransformWizardPage() = default;

bool te::qt::widgets::RasterTransformWizardPage::isComplete() const
{
  return true;
}

void te::qt::widgets::RasterTransformWizardPage::setLayer(te::map::AbstractLayerPtr layer)
{
  //list bands
  m_uiPageForm->m_bandsListWidget->clear();

  //get input raster
  std::unique_ptr<te::da::DataSet> ds = layer->getData();

  if(ds.get())
  {
    std::size_t rpos = te::da::GetFirstPropertyPos(ds.get(), te::dt::RASTER_TYPE);

    std::unique_ptr<te::rst::Raster> inputRst = ds->getRaster(rpos);

    if(inputRst.get())
    {
      m_layer = layer;

      for(unsigned int i = 0; i < inputRst->getNumberOfBands(); ++i)
      {
        m_uiPageForm->m_bandsListWidget->addItem(QString::number(i));
      }
    }
    else
    {
      QMessageBox::critical(
          this, tr("Warning"),
          tr("Invalid input layer"));
    }
  }
}

void te::qt::widgets::RasterTransformWizardPage::getSelectedInBandIndexes( 
  std::vector< unsigned int >& bandsIndexes ) const
{
  QList<QListWidgetItem *> selectedBandsItems =
    m_uiPageForm->m_bandsListWidget->selectedItems();
  
  QList<QListWidgetItem *>::const_iterator it = selectedBandsItems.begin();
  const QList<QListWidgetItem *>::const_iterator itEnd = selectedBandsItems.end();
  
  while( it != itEnd )
  {
    bandsIndexes.push_back( (*it)->text().toUInt() );
    ++it;
  }
  
  std::sort( bandsIndexes.begin(), bandsIndexes.end() );
}

std::vector< unsigned int > 
te::qt::widgets::RasterTransformWizardPage::getOutDataTypes() const
{
  std::vector< unsigned int > outDatatypes;
  
  std::vector< unsigned int > selectedBandsIndexes;
  getSelectedInBandIndexes( selectedBandsIndexes );
  
  std::unique_ptr<te::da::DataSet> ds;
  std::unique_ptr<te::rst::Raster> inputRst;
  if( getInRaster( ds, inputRst ) )
  {
    for(unsigned int selectedBandsIndexesIdx = 0; 
      selectedBandsIndexesIdx < selectedBandsIndexes.size() ;
      ++selectedBandsIndexesIdx)
    {
      if( 
          m_uiPageForm->m_dataTypeConvEnabledCheckBox->checkState()
          ==
          Qt::Checked 
        )
      {
        outDatatypes.push_back( m_uiPageForm->m_outputDatatypeComboBox->currentData( 
          Qt::UserRole ).toUInt() );
      }
      else
      {
        outDatatypes.push_back( inputRst->getBand( selectedBandsIndexes[
          selectedBandsIndexesIdx ] )->getProperty()->m_type );
      }
    }
  }  
  
  return outDatatypes;
}

unsigned int te::qt::widgets::RasterTransformWizardPage::getInterpolator() const
{
  return m_uiPageForm->m_interpComboBox->currentData( Qt::UserRole ).toUInt();
}

bool te::qt::widgets::RasterTransformWizardPage::getInRaster( 
  std::unique_ptr<te::da::DataSet>& inDataSet,
  std::unique_ptr<te::rst::Raster >& inRasterPtr ) const
{
  inDataSet.reset();
  inRasterPtr.reset();
 
  if( m_layer.get() == nullptr ) return false;
  
  inDataSet = m_layer->getData();
  if( inDataSet.get() == nullptr ) return false;

  std::size_t rpos = te::da::GetFirstPropertyPos(
    inDataSet.get(), te::dt::RASTER_TYPE);

  inRasterPtr = inDataSet->getRaster(rpos);
  if(inRasterPtr.get() == nullptr ) return false;
  
  return true;;
}

te::map::AbstractLayer* te::qt::widgets::RasterTransformWizardPage::getLayerPtr()
{
  return m_layer.get();
}


void te::qt::widgets::RasterTransformWizardPage::onSelectOldSRSPushButtonClicked()
{
  te::qt::widgets::SRSManagerDialog srsDialog(this);
  srsDialog.setWindowTitle(tr("Choose the Old SRS"));
  
  if(srsDialog.exec() == QDialog::Accepted)
  {
    std::pair<int, std::string> srid = srsDialog.getSelectedSRS();

    QString strSRID;
    strSRID.setNum(srid.first);
    m_uiPageForm->m_oldSRSLineEdit->setText(strSRID);
  }
}

void te::qt::widgets::RasterTransformWizardPage::onSelectNewSRSPushButtonClicked()
{
  te::qt::widgets::SRSManagerDialog srsDialog(this);
  srsDialog.setWindowTitle(tr("Choose the New SRS"));
  
  if(srsDialog.exec() == QDialog::Accepted)
  {
    std::pair<int, std::string> srid = srsDialog.getSelectedSRS();

    QString strSRID;
    strSRID.setNum(srid.first);
    m_uiPageForm->m_newSRSLineEdit->setText(strSRID);
  }
}


